@@ -9,11 +9,11 @@ import typing
 if typing.TYPE_CHECKING:
 	from plover.engine import StenoEngine  # type: ignore
 
-def show_search_dialog(engine: StenoEngine, argument: str)->None:
+def open_dialog(engine: StenoEngine, argument: str)->None:
 	from . import manager
-	manager.get().show(argument)
+	manager.get().open_dialog(argument)
 
-def close_window(engine: StenoEngine, argument: str)->None:
+def close_dialog(engine: StenoEngine, argument: str)->None:
 	assert not argument
 	from . import manager
-	manager.get().close_window()
+	manager.get().close_dialog()
