@@ -157,8 +157,11 @@ from .lib import text_to_outline
 def add_translation()->None:
 	assert state is WINDOW_OPEN or isinstance(state, Editing)
 
-	if not (dialog.output.text() and dialog.description.text()):
-		show_error("Output and description must be filled")
+	if not dialog.output.text():
+		show_error("Output must be filled")
+		return
+	if not (dialog.brief.text() or dialog.description.text()):
+		show_error("Brief or description must be filled")
 		return
 
 	new_entry=Entry(
@@ -168,17 +171,21 @@ def add_translation()->None:
 		)
 
 	if state is WINDOW_OPEN:
+		if not message.func.add_translation(new_entry):
+			show_error("Cannot add translation")
+			return
 		dialog.matches.insertRow(0)
+		dialog.refresh_all_vertical_header()
 		dialog.set_row_data(0, new_entry)
-		message.call.add_translation(new_entry)
 	else:
 		assert isinstance(state, Editing), state
 		old_entry=state.entry
+		if old_entry!=new_entry:
+			if not message.func.edit_translation(old_entry, new_entry):
+				show_error("Cannot edit translation")
+				return
 		dialog.set_row_data(state.row, new_entry)
 		set_state(WINDOW_OPEN)
-		if old_entry!=new_entry:
-			message.call.remove_translation(old_entry)
-			message.call.add_translation(new_entry)
 
 	dialog.output.setText("")
 	dialog.description.setFocus()
@@ -265,6 +272,7 @@ def repopulate_matches(query: str)->None:
 		return
 	result: List[Entry] = message.func.search(query)
 	dialog.matches.setRowCount(len(result))
+	dialog.refresh_all_vertical_header()
 	for row, entry in enumerate(result):
 		dialog.set_row_data(row, entry)
 
